from datetime import date
from threading import local

from django.utils.deprecation import MiddlewareMixin

from .abmain import AB
from .models import Experiment


_thread_locals = local()


def get_current_request():
    return getattr(_thread_locals, 'request', None)


def get_updated_traffic(exp):
    test_1, test_2 = exp.test_set.all()
    try:
        test_1_ratio = float(test_1.conversions) / test_1.hits
        test_2_ratio = float(test_2.conversions) / test_2.hits
        updated_traffic = int(test_2_ratio * 100/(test_1_ratio + test_2_ratio))
        return updated_traffic
    except ZeroDivisionError:
        return 50


def get_status(exp):
    start = exp.start
    end = exp.end
    if start > date.today() or end < date.today():
        active = False
    else:
        active = True
    return active


class ABMiddleware(MiddlewareMixin):
    def process_request(self, request):
        _thread_locals.request = request
        request.ab = AB(request)
        if request.ab.is_active():
            exps = Experiment.objects.all()
            for exp in exps:
                exp.is_active = get_status(exp)
                if exp.is_active:
                    exp.percentage = get_updated_traffic(exp)
                exp.save()
                if request.ab.is_converted(exp):
                    request.ab.convert(exp)



from __future__ import unicode_literals
import urlparse

from django.db import models
from django.conf import settings
from django.utils.translation import ugettext_lazy as _


class UrlMixin(models.Model):
    class Meta:
        abstract = True

    def get_url(self):
        if hasattr(self.get_url_path, "dont_recurse"):
            raise NotImplementedError
        try:
            path = self.get_url_path()
        except NotImplementedError:
            raise
        website_url = getattr(
            settings, "DEFAULT_WEBSITE_URL",
            "http://127.0.0.1:8000"
        )
        return website_url + path

    def get_url_path(self):
        if hasattr(self.get_url, "dont_recurse"):
            raise NotImplementedError
        try:
            url = self.get_url()
        except NotImplementedError:
            raise
        bits = urlparse.urlparse(url)
        return urlparse.urlunparse(("", "") + bits[2:])
    get_url_path.dont_recurse = True

    def get_absolute_url(self):
        return self.get_url_path()


class CreationModificationDateMixin(models.Model):
    created = models.DateTimeField(
        _("Creation Date and Time"),
        editable=False,
        auto_now_add=True,
    )
    modified = models.DateTimeField(
        _("Modification Date and Time"),
        null=True,
        editable=False,
        auto_now=True,
    )

    def save(self, *args, **kwargs):
        super(CreationModificationDateMixin, self).save(*args, **kwargs)
    save.alters_data = True

    class Meta:
        abstract = True



from __future__ import unicode_literals
from datetime import date

from django.core.urlresolvers import reverse
from django.core.validators import MaxValueValidator
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from .utils.models import UrlMixin, CreationModificationDateMixin


@python_2_unicode_compatible
class Experiment(CreationModificationDateMixin, UrlMixin):
    name = models.CharField(max_length=255, unique=True)
    template_name = models.CharField(max_length=255, unique=True,
                                     help_text=_("Example : '/main/signup.html'. Original Template"))
    goal = models.CharField(max_length=255, unique=True,
                            help_text=_("Example : '/main/signup_complete'. The path on goal Completion"))
    start = models.DateField(blank=False, db_index=True,
                             help_text=_("This Test starts at UTC +00:00 at the selected Date."))
    end = models.DateField(blank=False, null=True,
                           help_text=_("This Test ends at UTC +00:00 at the selected Date."))
    percentage = models.PositiveIntegerField(_("Traffic Percentage"), validators=[MaxValueValidator(100), ], default=50,
                                             help_text=_("Percentage of traffic to redirect to Variant B"))
    is_active = models.BooleanField(_("Active"), default=False)

    class Meta:
        verbose_name = _("Experiment")
        verbose_name_plural = _("Experiments")

    def __str__(self):
        return self.name

    def get_url_path(self):
        return reverse(
            "experiment_details", kwargs={
                    "experiment_id": str(self.pk)
                }
        )

    def save(self, *args, **kwargs):
        if self.start > date.today() or self.end < date.today():
            self.is_active = False
        super(Experiment, self).save(*args, **kwargs)


@python_2_unicode_compatible
class Test(models.Model):
    experiment = models.ForeignKey(Experiment)
    template_name = models.CharField(max_length=255, unique=True,
                                     help_text=_("Example : '/main/signup1.html'. Template to be Tested"))
    hits = models.IntegerField(blank=True, default=0,
                               help_text=_("# Uniques who have seen this test."))
    conversions = models.IntegerField(blank=True, default=0,
                                      help_text=_("# Uniques that have reached the goal from this test."))

    class Meta:
        verbose_name = _("Test")
        verbose_name_plural = _("Tests")

    def __str__(self):
        return self.template_name



import random

from .models import Experiment, Test


class AB(object):

    def __init__(self, request):
        self.request = request

    def is_active(self):
        return "active" in self.request.session

    def is_converted(self, exp):
        return self.is_experiment_active(exp) and not self.is_experiment_converted(exp) \
                    and exp.goal in self.request.path

    def is_experiment_active(self, exp):
        return self.get_experiment_key(exp) in self.request.session

    def is_experiment_converted(self, exp):
        return "converted" in self.request.session[self.get_experiment_key(exp)]

    def get_traffic(self, percentage):
        return 0 if random.random() <= percentage else 1

    def get_test(self, exp):
        tests = exp.test_set.all()
        test_index = self.get_traffic(exp.percentage / 100.0)
        print test_index
        test = tests[test_index]
        return test

    def get_experiment_key(self, exp):
        return "ab_exp_key_%s" % exp.name

    def get_experiment(self, template_name):
        return Experiment.objects.get(template_name=template_name)

    def run(self, template_name):
        try:
            exp = self.get_experiment(template_name)
        except Experiment.DoesNotExist:
            return template_name

        if not exp.is_active:
            return template_name

        key = self.get_experiment_key(exp)
        if self.is_experiment_active(exp):
            return self.request.session[key]["template"]

        test = self.get_test(exp)
        self.activate(test, key)

        return test.template_name

    def activate(self, test, key):
        test.hits += 1
        test.save()
        print "==========================="
        print "Hits Updated"
        print "==========================="
        self.request.session[key] = {"id": test.id, "template": test.template_name}

        self.request.session["active"] = True

    def convert(self, exp):
        key = self.get_experiment_key(exp)
        test_id = self.request.session[key]["id"]
        test = Test.objects.get(pk=test_id)
        test.conversions += 1
        test.save()
        print "==========================="
        print "Conversions Updated"
        print "==========================="

        self.request.session[key]["converted"] = True
        self.request.session.modified = True



# _*_ coding: utf-8 _*_
from distutils.core import setup
from setuptools import find_packages

setup(
    name='django-abtoast',
    version='1.0.3',
    author='Hiten Sharma',
    author_email='sharmahiten3@gmail.com',
    packages=find_packages(),
    url='https://github.com/htadg/ABToast',
    license='MIT License',
    description='ABToast is an A/B Testing app that is developed in django.',
    long_description=open('README.md').read(),
    zip_safe=False,
)



from django.contrib import admin

from .models import Experiment, Test


class TestInline(admin.TabularInline):
    model = Test
    max_num = 2
    extra = 2


class ExperimentAdmin(admin.ModelAdmin):
    inlines = (TestInline, )
    list_display = ('name', 'start', 'end', 'is_active', )
    # exclude = ('status', )
    search_fields = ('name', )
    list_filter = ('start', 'end', 'is_active', )
    ordering = ('-start', )

admin.site.register(Experiment, ExperimentAdmin)
